package javax.swing.plaf.synth;

import java.util.ArrayList;
import java.awt.Component;
import java.util.List;
import javax.swing.ButtonModel;
import sun.swing.MenuItemLayoutHelper;
import javax.swing.JMenuItem;
import java.awt.Color;
import javax.swing.JComponent;
import javax.swing.text.View;
import java.awt.Dimension;
import sun.swing.SwingUtilities2;
import java.awt.Font;
import javax.swing.SwingUtilities;
import sun.swing.plaf.synth.SynthIcon;
import javax.swing.Icon;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;

public class SynthGraphicsUtils
{
    private Rectangle paintIconR;
    private Rectangle paintTextR;
    private Rectangle paintViewR;
    private Insets paintInsets;
    private Rectangle iconR;
    private Rectangle textR;
    private Rectangle viewR;
    private Insets viewSizingInsets;
    
    public SynthGraphicsUtils() {
        this.paintIconR = new Rectangle();
        this.paintTextR = new Rectangle();
        this.paintViewR = new Rectangle();
        this.paintInsets = new Insets(0, 0, 0, 0);
        this.iconR = new Rectangle();
        this.textR = new Rectangle();
        this.viewR = new Rectangle();
        this.viewSizingInsets = new Insets(0, 0, 0, 0);
    }
    
    public void drawLine(SynthContext synthContext, Object o, Graphics graphics, int n, int n2, int n3, int n4) {
        graphics.drawLine(n, n2, n3, n4);
    }
    
    public void drawLine(SynthContext synthContext, Object o, Graphics graphics, int i, int j, int n, int n2, Object anObject) {
        if ("dashed".equals(anObject)) {
            if (i == n) {
                for (j = (j += j % 2); j <= n2; j += 2) {
                    graphics.drawLine(i, j, n, j);
                }
            }
            else if (j == n2) {
                for (i = (i += i % 2); i <= n; i += 2) {
                    graphics.drawLine(i, j, i, n2);
                }
            }
        }
        else {
            this.drawLine(synthContext, o, graphics, i, j, n, n2);
        }
    }
    
    public String layoutText(SynthContext synthContext, FontMetrics fontMetrics, String s, Icon icon, int n, int n2, int n3, int n4, Rectangle rectangle, Rectangle rectangle2, Rectangle rectangle3, int n5) {
        if ((icon instanceof SynthIcon)) {
            SynthGraphicsUtils.SynthIconWrapper value = SynthGraphicsUtils.SynthIconWrapper.get((SynthIcon)icon, synthContext);
            String layoutCompoundLabel = SwingUtilities.layoutCompoundLabel(synthContext.getComponent(), fontMetrics, s, value, n2, n, n4, n3, rectangle, rectangle2, rectangle3, n5);
            SynthGraphicsUtils.SynthIconWrapper.release(value);
            return layoutCompoundLabel;
        }
        return SwingUtilities.layoutCompoundLabel(synthContext.getComponent(), fontMetrics, s, icon, n2, n, n4, n3, rectangle, rectangle2, rectangle3, n5);
    }
    
    public int computeStringWidth(SynthContext synthContext, Font font, FontMetrics fm, String string) {
        return SwingUtilities2.stringWidth(synthContext.getComponent(), fm, string);
    }
    
    public Dimension getMinimumSize(SynthContext synthContext, Font font, String s, Icon icon, int n, int n2, int n3, int n4, int n5, int n6) {
        JComponent component = synthContext.getComponent();
        Dimension preferredSize = this.getPreferredSize(synthContext, font, s, icon, n, n2, n3, n4, n5, n6);
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            preferredSize.width = (int)((float)preferredSize.width - (view.getPreferredSpan(0) - view.getMinimumSpan(0)));
            return preferredSize;
        }
        return preferredSize;
    }
    
    public Dimension getMaximumSize(SynthContext synthContext, Font font, String s, Icon icon, int n, int n2, int n3, int n4, int n5, int n6) {
        JComponent component = synthContext.getComponent();
        Dimension preferredSize = this.getPreferredSize(synthContext, font, s, icon, n, n2, n3, n4, n5, n6);
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            preferredSize.width = (int)((float)preferredSize.width + (view.getMaximumSpan(0) - view.getPreferredSpan(0)));
            return preferredSize;
        }
        return preferredSize;
    }
    
    public int getMaximumCharHeight(SynthContext synthContext) {
        FontMetrics fontMetrics = synthContext.getComponent().getFontMetrics(synthContext.getStyle().getFont(synthContext));
        return fontMetrics.getAscent() + fontMetrics.getDescent();
    }
    
    public Dimension getPreferredSize(SynthContext synthContext, Font font, String s, Icon icon, int n, int n2, int n3, int n4, int n5, int n6) {
        JComponent component = synthContext.getComponent();
        Insets insets = component.getInsets(this.viewSizingInsets);
        int n7 = insets.left + insets.right;
        int n8 = insets.top + insets.bottom;
        if (icon == null && (s == null || font == null)) {
            return new Dimension(n7, n8);
        }
        if (s == null || (icon != null && font == null)) {
            return new Dimension(SynthIcon.getIconWidth(icon, synthContext) + n7, SynthIcon.getIconHeight(icon, synthContext) + n8);
        }
        FontMetrics fontMetrics = component.getFontMetrics(font);
        Rectangle iconR = this.iconR;
        Rectangle iconR2 = this.iconR;
        Rectangle iconR3 = this.iconR;
        Rectangle iconR4 = this.iconR;
        int n9 = 0;
        iconR4.height = n9;
        iconR3.width = n9;
        iconR2.y = n9;
        iconR.x = n9;
        Rectangle textR = this.textR;
        Rectangle textR2 = this.textR;
        Rectangle textR3 = this.textR;
        Rectangle textR4 = this.textR;
        int n10 = 0;
        textR4.height = n10;
        textR3.width = n10;
        textR2.y = n10;
        textR.x = n10;
        this.viewR.x = n7;
        this.viewR.y = n8;
        Rectangle viewR = this.viewR;
        Rectangle viewR2 = this.viewR;
        int n11 = 32767;
        viewR2.height = n11;
        viewR.width = n11;
        this.layoutText(synthContext, fontMetrics, s, icon, n, n2, n3, n4, this.viewR, this.iconR, this.textR, n5);
        Dimension dimension = new Dimension(Math.max(this.iconR.x + this.iconR.width, this.textR.x + this.textR.width) - Math.min(this.iconR.x, this.textR.x), Math.max(this.iconR.y + this.iconR.height, this.textR.y + this.textR.height) - Math.min(this.iconR.y, this.textR.y));
        dimension.width += n7;
        dimension.height += n8;
        return dimension;
    }
    
    public void paintText(SynthContext synthContext, Graphics graphics, String s, Rectangle rectangle, int n) {
        this.paintText(synthContext, graphics, s, rectangle.x, rectangle.y, n);
    }
    
    public void paintText(SynthContext synthContext, Graphics graphics, String text, int x, int y, int underlinedIndex) {
        if (text != null) {
            JComponent component = synthContext.getComponent();
            y += SwingUtilities2.getFontMetrics(component, graphics).getAscent();
            SwingUtilities2.drawStringUnderlineCharAt(component, graphics, text, underlinedIndex, x, y);
        }
    }
    
    public void paintText(SynthContext synthContext, Graphics g, String s, Icon icon, int n, int n2, int n3, int n4, int n5, int n6, int n7) {
        if (icon == null && s == null) {
            return;
        }
        JComponent component = synthContext.getComponent();
        FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(component, g);
        Insets paintingInsets = SynthLookAndFeel.getPaintingInsets(synthContext, this.paintInsets);
        this.paintViewR.x = paintingInsets.left;
        this.paintViewR.y = paintingInsets.top;
        this.paintViewR.width = component.getWidth() - (paintingInsets.left + paintingInsets.right);
        this.paintViewR.height = component.getHeight() - (paintingInsets.top + paintingInsets.bottom);
        Rectangle paintIconR = this.paintIconR;
        Rectangle paintIconR2 = this.paintIconR;
        Rectangle paintIconR3 = this.paintIconR;
        Rectangle paintIconR4 = this.paintIconR;
        int n8 = 0;
        paintIconR4.height = n8;
        paintIconR3.width = n8;
        paintIconR2.y = n8;
        paintIconR.x = n8;
        Rectangle paintTextR = this.paintTextR;
        Rectangle paintTextR2 = this.paintTextR;
        Rectangle paintTextR3 = this.paintTextR;
        Rectangle paintTextR4 = this.paintTextR;
        int n9 = 0;
        paintTextR4.height = n9;
        paintTextR3.width = n9;
        paintTextR2.y = n9;
        paintTextR.x = n9;
        String layoutText = this.layoutText(synthContext, fontMetrics, s, icon, n, n2, n3, n4, this.paintViewR, this.paintIconR, this.paintTextR, n5);
        if (icon != null) {
            Color color = g.getColor();
            if (synthContext.getStyle().getBoolean(synthContext, "TableHeader.alignSorterArrow", false) && "TableHeader.renderer".equals(component.getName())) {
                this.paintIconR.x = this.paintViewR.width - this.paintIconR.width;
            }
            else {
                Rectangle paintIconR5 = this.paintIconR;
                paintIconR5.x += n7;
            }
            Rectangle paintIconR6 = this.paintIconR;
            paintIconR6.y += n7;
            SynthIcon.paintIcon(icon, synthContext, g, this.paintIconR.x, this.paintIconR.y, this.paintIconR.width, this.paintIconR.height);
            g.setColor(color);
        }
        if (s != null) {
            View view = (View)component.getClientProperty("html");
            if (view != null) {
                view.paint(g, this.paintTextR);
            }
            else {
                Rectangle paintTextR5 = this.paintTextR;
                paintTextR5.x += n7;
                Rectangle paintTextR6 = this.paintTextR;
                paintTextR6.y += n7;
                this.paintText(synthContext, g, layoutText, this.paintTextR, n6);
            }
        }
        return;
    }
    
    static Dimension getPreferredMenuItemSize(SynthContext synthContext, SynthContext synthContext2, JComponent component, Icon icon, Icon icon2, int n, String s, boolean b, String s2) {
        JMenuItem menuItem = (JMenuItem)component;
        SynthMenuItemLayoutHelper synthMenuItemLayoutHelper = new SynthMenuItemLayoutHelper(synthContext, synthContext2, menuItem, icon, icon2, MenuItemLayoutHelper.createMaxRect(), n, s, SynthLookAndFeel.isLeftToRight(menuItem), b, s2);
        Dimension result = new Dimension();
        int gap = synthMenuItemLayoutHelper.getGap();
        result.width = 0;
        MenuItemLayoutHelper.addMaxWidth(synthMenuItemLayoutHelper.getCheckSize(), gap, result);
        MenuItemLayoutHelper.addMaxWidth(synthMenuItemLayoutHelper.getLabelSize(), gap, result);
        MenuItemLayoutHelper.addWidth(synthMenuItemLayoutHelper.getMaxAccOrArrowWidth(), 5 * gap, result);
        result.width -= gap;
        result.height = MenuItemLayoutHelper.max(new int[] { synthMenuItemLayoutHelper.getCheckSize().getHeight(), synthMenuItemLayoutHelper.getLabelSize().getHeight(), synthMenuItemLayoutHelper.getAccSize().getHeight(), synthMenuItemLayoutHelper.getArrowSize().getHeight() });
        Insets insets = synthMenuItemLayoutHelper.getMenuItem().getInsets();
        if (insets != null) {
            result.width += insets.left + insets.right;
            result.height += insets.top + insets.bottom;
        }
        if (result.width % 2 == 0) {
            ++result.width;
        }
        if (result.height % 2 == 0) {
            ++result.height;
            return result;
        }
        return result;
    }
    
    static void applyInsets(Rectangle rectangle, Insets insets, boolean b) {
        if (insets != null) {
            rectangle.x += (b ? insets.left : insets.right);
            rectangle.y += insets.top;
            rectangle.width -= (b ? insets.right : insets.left) + rectangle.x;
            rectangle.height -= insets.bottom + rectangle.y;
        }
    }
    
    static void paint(SynthContext synthContext, SynthContext synthContext2, Graphics graphics, Icon icon, Icon icon2, String s, int n, String s2) {
        JMenuItem menuItem = (JMenuItem)synthContext.getComponent();
        graphics.setFont(synthContext.getStyle().getFont(synthContext));
        Rectangle rectangle = new Rectangle(0, 0, menuItem.getWidth(), menuItem.getHeight());
        int leftToRight = SynthLookAndFeel.isLeftToRight(menuItem);
        applyInsets(rectangle, menuItem.getInsets(), (boolean)leftToRight);
        SynthMenuItemLayoutHelper synthMenuItemLayoutHelper = new SynthMenuItemLayoutHelper(synthContext, synthContext2, menuItem, icon, icon2, rectangle, n, s, (boolean)leftToRight, MenuItemLayoutHelper.useCheckAndArrow(menuItem), s2);
        paintMenuItem(graphics, synthMenuItemLayoutHelper, synthMenuItemLayoutHelper.layoutMenuItem());
    }
    
    static void paintMenuItem(Graphics graphics, SynthMenuItemLayoutHelper synthMenuItemLayoutHelper, MenuItemLayoutHelper.LayoutResult layoutResult) {
        Font font = graphics.getFont();
        Color color = graphics.getColor();
        paintCheckIcon(graphics, synthMenuItemLayoutHelper, layoutResult);
        paintIcon(graphics, synthMenuItemLayoutHelper, layoutResult);
        paintText(graphics, synthMenuItemLayoutHelper, layoutResult);
        paintAccText(graphics, synthMenuItemLayoutHelper, layoutResult);
        paintArrowIcon(graphics, synthMenuItemLayoutHelper, layoutResult);
        graphics.setColor(color);
        graphics.setFont(font);
    }
    
    static void paintBackground(Graphics graphics, SynthMenuItemLayoutHelper synthMenuItemLayoutHelper) {
        paintBackground(synthMenuItemLayoutHelper.getContext(), graphics, synthMenuItemLayoutHelper.getMenuItem());
    }
    
    static void paintBackground(SynthContext synthContext, Graphics graphics, JComponent component) {
        synthContext.getPainter().paintMenuItemBackground(synthContext, graphics, 0, 0, component.getWidth(), component.getHeight());
    }
    
    static void paintIcon(Graphics graphics, SynthMenuItemLayoutHelper synthMenuItemLayoutHelper, MenuItemLayoutHelper.LayoutResult layoutResult) {
        if (synthMenuItemLayoutHelper.getIcon() != null) {
            JMenuItem menuItem = synthMenuItemLayoutHelper.getMenuItem();
            ButtonModel model = menuItem.getModel();
            Icon icon;
            if (!model.isEnabled()) {
                icon = menuItem.getDisabledIcon();
            }
            else if (model.isPressed() && model.isArmed()) {
                icon = menuItem.getPressedIcon();
                if (icon == null) {
                    icon = menuItem.getIcon();
                }
            }
            else {
                icon = menuItem.getIcon();
            }
            if (icon != null) {
                Rectangle iconRect = layoutResult.getIconRect();
                SynthIcon.paintIcon(icon, synthMenuItemLayoutHelper.getContext(), graphics, iconRect.x, iconRect.y, iconRect.width, iconRect.height);
            }
        }
    }
    
    static void paintCheckIcon(Graphics graphics, SynthMenuItemLayoutHelper synthMenuItemLayoutHelper, MenuItemLayoutHelper.LayoutResult layoutResult) {
        if (synthMenuItemLayoutHelper.getCheckIcon() != null) {
            Rectangle checkRect = layoutResult.getCheckRect();
            SynthIcon.paintIcon(synthMenuItemLayoutHelper.getCheckIcon(), synthMenuItemLayoutHelper.getContext(), graphics, checkRect.x, checkRect.y, checkRect.width, checkRect.height);
        }
    }
    
    static void paintAccText(Graphics graphics, SynthMenuItemLayoutHelper synthMenuItemLayoutHelper, MenuItemLayoutHelper.LayoutResult layoutResult) {
        String accText = synthMenuItemLayoutHelper.getAccText();
        if (accText != null && !accText.equals("")) {
            graphics.setColor(synthMenuItemLayoutHelper.getAccStyle().getColor(synthMenuItemLayoutHelper.getAccContext(), ColorType.TEXT_FOREGROUND));
            graphics.setFont(synthMenuItemLayoutHelper.getAccStyle().getFont(synthMenuItemLayoutHelper.getAccContext()));
            synthMenuItemLayoutHelper.getAccGraphicsUtils().paintText(synthMenuItemLayoutHelper.getAccContext(), graphics, accText, layoutResult.getAccRect().x, layoutResult.getAccRect().y, -1);
        }
    }
    
    static void paintText(Graphics graphics, SynthMenuItemLayoutHelper synthMenuItemLayoutHelper, MenuItemLayoutHelper.LayoutResult layoutResult) {
        if (!synthMenuItemLayoutHelper.getText().equals("")) {
            if (synthMenuItemLayoutHelper.getHtmlView() != null) {
                synthMenuItemLayoutHelper.getHtmlView().paint(graphics, layoutResult.getTextRect());
            }
            else {
                graphics.setColor(synthMenuItemLayoutHelper.getStyle().getColor(synthMenuItemLayoutHelper.getContext(), ColorType.TEXT_FOREGROUND));
                graphics.setFont(synthMenuItemLayoutHelper.getStyle().getFont(synthMenuItemLayoutHelper.getContext()));
                synthMenuItemLayoutHelper.getGraphicsUtils().paintText(synthMenuItemLayoutHelper.getContext(), graphics, synthMenuItemLayoutHelper.getText(), layoutResult.getTextRect().x, layoutResult.getTextRect().y, synthMenuItemLayoutHelper.getMenuItem().getDisplayedMnemonicIndex());
            }
        }
    }
    
    static void paintArrowIcon(Graphics graphics, SynthMenuItemLayoutHelper synthMenuItemLayoutHelper, MenuItemLayoutHelper.LayoutResult layoutResult) {
        if (synthMenuItemLayoutHelper.getArrowIcon() != null) {
            Rectangle arrowRect = layoutResult.getArrowRect();
            SynthIcon.paintIcon(synthMenuItemLayoutHelper.getArrowIcon(), synthMenuItemLayoutHelper.getContext(), graphics, arrowRect.x, arrowRect.y, arrowRect.width, arrowRect.height);
        }
    }
    
    private static class SynthIconWrapper implements Icon
    {
        private static final List<SynthGraphicsUtils.SynthIconWrapper> CACHE;
        private SynthIcon synthIcon;
        private SynthContext context;
        
        static SynthGraphicsUtils.SynthIconWrapper get(SynthIcon synthIcon, SynthContext synthContext) {
            synchronized (SynthGraphicsUtils.SynthIconWrapper.CACHE) {
                int size = SynthGraphicsUtils.SynthIconWrapper.CACHE.size();
                if (size > 0) {
                    SynthGraphicsUtils.SynthIconWrapper synthIconWrapper = (SynthGraphicsUtils.SynthIconWrapper)SynthGraphicsUtils.SynthIconWrapper.CACHE.remove(size - 1);
                    synthIconWrapper.reset(synthIcon, synthContext);
                    return synthIconWrapper;
                }
            }
            return new SynthGraphicsUtils.SynthIconWrapper(synthIcon, synthContext);
        }
        
        static void release(SynthGraphicsUtils.SynthIconWrapper synthIconWrapper) {
            synthIconWrapper.reset(null, null);
            synchronized (SynthGraphicsUtils.SynthIconWrapper.CACHE) {
                SynthGraphicsUtils.SynthIconWrapper.CACHE.add(synthIconWrapper);
            }
        }
        
        SynthIconWrapper(SynthIcon synthIcon, SynthContext synthContext) {
            this.reset(synthIcon, synthContext);
        }
        
        void reset(SynthIcon synthIcon, SynthContext context) {
            this.synthIcon = synthIcon;
            this.context = context;
        }
        
        @Override
        public void paintIcon(Component component, Graphics graphics, int n, int n2) {
        }
        
        @Override
        public int getIconWidth() {
            return this.synthIcon.getIconWidth(this.context);
        }
        
        @Override
        public int getIconHeight() {
            return this.synthIcon.getIconHeight(this.context);
        }
        
        static {
            CACHE = new ArrayList(1);
        }
    }
}
